%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%                                                                   %%%
%%%	MATLAB Code for:					      									 %%% 
%%%								      												 %%%
%%%			GENERATIONAL ACCOUNTING			      							 %%%
%%%								      												 %%%
%%%								      												 %%%
%%%	Programming by:  	Philip Oreopoulos		      						 %%%
%%%								      												 %%%
%%%	Filename:		gamatold.m			      								 %%%
%%%								      												 %%%
%%%								      												 %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%==================================%
%  Parameter Settings and Switches %
%==================================%


switch01 = 0;				% omits cap. tax. adj. if '1'
switch02 = 0;				% also calculates m & f together
switch05 = 0;				% equalize burdens experiment
switch06 = 2;				% specifies aggregate projections
								% 1=low,2=med,3=high,4=with edu.
switch07 = 2;				% specifies demographic projections
								% (1=low, 2=med, 3=high)
                        
r=0.060;	
g=0.012;						% note: adjust growth rates below
ages=101;     				
types=17;    				% note: types 16, 17 are govcon
			
baseyear 	= 1995;			
lastyear 	= 2100;			
projyear 	= 2070;			

exchangerate = 1.0;		

											% these definitions make the notation
prjyear=projyear-baseyear+1;		% easier to follow when working
lstyear=lastyear-baseyear+1;		% with column and row numbers
remainyear=lastyear-projyear+1;



%=====================================%
% Load in data files and conform data %
%=====================================%

if switch07 == 1
   load POPMLOW.txt;POPM=POPMLOW;clear POPMLOW;
   load POPFLOW.txt;POPF=POPFLOW;clear POPFLOW;
elseif switch07 == 2
   load POPMMED.txt;POPM=POPMMED;clear POPMMED;
   load POPFMED.txt;POPF=POPFMED;clear POPFMED;
elseif switch07 == 3
   load POPMHIGH.txt;POPM=POPMHIGH;clear POPMHIGH;
   load POPFHIGH.txt;POPF=POPFHIGH;clear POPFHIGH;
end

POPM(:,1)=[];		
POPF(:,1)=[];

POP(:,:,:,1)=POPM';	%males and females (indexed by 1 and 2)
POP(:,:,:,2)=POPF';	
POP=repmat(POP,[1,1,types,1]);		% POP(age,year,type,sex)


load RPM.txt; 		% loads relative profile data	
load RPF.txt;		% indexed by TYPE x AGE
RPM(:,1:2)=[];		% 1st col type, 2nd col year are removed
RPF(:,1:2)=[];		


RPM(14,1)=0;RPF(14,1)=0;
for i=2:ages;
   RPM(14,i)=RPM(14,i)-RPM(14,i-1);
   RPF(14,i)=RPF(14,i)-RPF(14,i-1);
end;
rm40=RPM(14,40);
for i=1:ages;
   RPM(14,i)=RPM(14,i)/rm40;
   RPF(14,i)=RPF(14,i)/rm40;
end;



RP(:,:,:,1) = repmat(permute(RPM,[2,3,1,4]),[1,prjyear,1,1]);
RP(:,:,:,2) = repmat(permute(RPF,[2,3,1,4]),[1,prjyear,1,1]);

			% RP matrix is relative profiles with
			% dims RP(age,year,type,sex), and the
			% year goes from baseyear to projyear  

if switch06 == 1
   load AGGLO.txt; AGG=AGGLO;clear AGGLO;g=0.07;
elseif switch06 == 2
   load AGGMED.txt; AGG=AGGMED;clear AGGMED;%g=0.012;
elseif switch06 == 3
   load AGGHI.txt; AGG=AGGHI;clear AGGHI;g=0.017;
elseif switch06 == 4
   load AGGEDUC.txt; AGG=AGGEDUC;clear AGGEDUC;g=0.012;
end

				      	% Aggregate matrix has dimensions
AGG(:,1)=[];			% YEAR x TYPE. first clumn
							% displaying the year is removed
							% NOTE: the last two columns contain
							% the values of federal and state government
							% consumption
 


%---------------------------------------------------------------%
% Gets discount and growth matricies with dims AGE x YEAR, from %
% baseyear to lastyear: (1+g)^0, (1+g)^1, (1+g)^2, ...          %
%---------------------------------------------------------------%

for i = 1:lstyear,DSCT(:,i) = (1/(1+r)^(i-1))*ones(ages,1);,end

DISCOUNTMX = repmat(DSCT(:,1:ages),[1,1,types,2]);


for j = 1:lstyear,GRTH(:,j) = (1+g)^(j-1)*ones(ages,1);,end

GROWTHMX = repmat(GRTH(:,1:ages-prjyear+1),[1,1,types,2]);


%==================================================%
% Converts Aggregates to billions of U.S. dollars  %
% and defines some variables			   				%
%==================================================%

AGG = exchangerate*1000000000*AGG;

DEBT = 2078610000000;

%===============================================%
%																%
% Capital Income Tax Adjustment (If switch01=0) %
%																%			
%===============================================%

qcap 	= 0.11100;
delcap 	= 0.00111;

if switch01 == 1, AGG(:,13) = 0;, end;

for i = 1:prjyear;
   AGG(i,12) = AGG(i,12) - (delcap*AGG(1,13))*((1+g)^(i-1));
end;

AGG(:,13) = qcap*AGG(:,13);

%=======================================================%
%																		  %
% Calculates from data per capita amounts of taxes paid %
% and transfers received for each age group by year     %
% 																		  %
%        PERCAP matrix is (age,year,type,sex)			  %
%																		  %
%=======================================================%

SUMS = RPM*POPM(1:prjyear,:)' + RPF*POPF(1:prjyear,:)'; 

UNITIES = AGG./SUMS';

PERCAP = RP.*repmat(permute(UNITIES,[3,1,2]),[ages,1,1,2]);

PERCAP(:,prjyear+1:ages,:,:) = ...
		repmat(PERCAP(:,prjyear,:,:),[1,ages-prjyear,1,1]) .*  ...
 		GROWTHMX(:,2:end,:,:);

%==================================================%
%						   									   %
% converts values to present value, and calculates %
% generational accounts for living age groups      %
%						   									   %
%==================================================%

PVTOTALS = PERCAP .* POP(:,1:ages,:,:) .* DISCOUNTMX;


for type = 1:types-2, for age = 1:ages

   TaxTransfers(age,type,1) = ...
	sum(diag(PVTOTALS(:,:,type,1),-(age-1)))./POP(age,1,1,1);
   TaxTransfers(age,type,2) = ...
	sum(diag(PVTOTALS(:,:,type,2),-(age-1)))./POP(age,1,1,2);

end, end

GA(:,1) = sum(TaxTransfers(:,:,1),2);	% sums along the rows in AVERAGES
GA(:,2) = sum(TaxTransfers(:,:,2),2);

%==============================================================%
%							       												%
% calculate total future PV spending on government consumption %
%							       												%
%==============================================================%

% GROWTH matrix is designed to increase per capital values at g,
% starting only at the end of the projection years

GROWTH = [ones(ages,prjyear) GRTH(:,2:remainyear)];

PERCAPGOV(:,1:prjyear,:,:) = PERCAP(:,1:prjyear,16:17,:);

PERCAPGOV(:,prjyear:lstyear,:,:) = ...
		repmat(PERCAP(:,prjyear,16:17,:),[1,remainyear,1,1]);

PVGOVTOTALS = PERCAPGOV .* POP(:,1:lstyear,16:17,:).* ...
	      repmat(DSCT,[1,1,2,2]).*repmat(GROWTH,[1,1,2,2]);
      
lastpart = PERCAPGOV(:,lstyear,:,:) .*...
              POP(:,lstyear,16:17,:).*...
              repmat(GROWTH(:,lstyear,:,:),[1,1,2,2]);

PVGOVCON = sum(PVGOVTOTALS(:)) + ...
   sum(lastpart(:))* ...
   (1+g)*(1/(1-((1+g)/(1+r))))*DSCT(1,lstyear)/(1+r);

      

				% sums every component of PVGOVTOTALS
				% and then computes out to infinity

%==============================================================%
%							       												%
% Computes residual (GOVGAP) from the intertemporal budget     %
% constraint which is assumed to be born by future generations %
%							       												%
%==============================================================%

GOVGAP=-PVGOVCON+DEBT-GA(:,1)'*POP(:,1,1,1)-GA(:,2)'*POP(:,1,1,2);

%===========================%
%			    					 %
% Adjusts future population %
%		            			 %
%===========================%

MFRATIO = GA(1,2)/GA(1,1); 

PartialPOPADJ = ...
   [POP(1,2:lstyear,1,1) + MFRATIO*POP(1,2:lstyear,1,2)].*...
   DSCT(1,1:lstyear-1).*GRTH(1,1:lstyear-1);

PINF = (POP(1,lstyear,1,1).*DSCT(1,end).*GRTH(1,end) + ...
   MFRATIO*POP(1,lstyear,1,2).*DSCT(1,end).*GRTH(1,end)).*...
   (1/(1-((1+g)/(1+r))));

POPADJ = (sum(PartialPOPADJ) + PINF)/(1+r);

%=======================================%
%												  	 %
% computes future generational accounts %
%													 %
%=======================================%


GAFUTURE(1) = GOVGAP/POPADJ;
GAFUTURE(2) = MFRATIO*(GOVGAP/POPADJ);


%================================================================%
% calculates GAs with males and females together if switch02 = 1 %
%================================================================%

if switch02 == 1
MFTOTALS = sum(PVTOTALS,4);
MFPOP = sum(POP(:,1,1,:),4);

for type = 1:types-2;
for age = 1:ages;

TaxTransfers(age,type,3) = ...
sum(diag(MFTOTALS(:,:,type),-(age-1)))./MFPOP(age,1);

end;
end;

PartPOPADJ = ...
   sum(POP(1,2:lstyear,1,:),4).*...
   DSCT(1,1:lstyear-1).*GRTH(1,1:lstyear-1);

PINFMF = [sum(POP(1,lstyear,1,:),4).*...
   DSCT(1,end).*GRTH(1,end)].*(1/(1-((1+g)/(1+r))));

POPADJMF = (sum(PartPOPADJ) + PINFMF)/(1+r);

GA(:,3) = sum(TaxTransfers(:,:,3),2);
GAFUTURE(3) = (-PVGOVCON+DEBT-GA(:,3)'*MFPOP)/POPADJMF;


end

%========================================%
% prints male GAs by 5 year age category %
%========================================%

format bank
GA(1:5:91,1)/1000
GAFUTURE(1,1)/1000
